/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: svdglev.hxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: vg $ $Date: 2007/04/11 16:20:33 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#ifndef _SVDGLEV_HXX
#define _SVDGLEV_HXX

#ifndef INCLUDED_SVXDLLAPI_H
#include "svx/svxdllapi.h"
#endif

#ifndef _SVDPOEV_HXX
#include <svx/svdpoev.hxx>
#endif

//************************************************************
//   Vorausdeklarationen
//************************************************************

class SdrGluePoint;

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
//
//   @@@@  @@    @@  @@ @@@@@   @@@@@ @@@@@  @@ @@@@@@  @@ @@ @@ @@@@@ @@   @@
//  @@  @@ @@    @@  @@ @@      @@    @@  @@ @@   @@    @@ @@ @@ @@    @@   @@
//  @@     @@    @@  @@ @@      @@    @@  @@ @@   @@    @@ @@ @@ @@    @@ @ @@
//  @@ @@@ @@    @@  @@ @@@@    @@@@  @@  @@ @@   @@    @@@@@ @@ @@@@  @@@@@@@
//  @@  @@ @@    @@  @@ @@      @@    @@  @@ @@   @@     @@@  @@ @@    @@@@@@@
//  @@  @@ @@    @@  @@ @@      @@    @@  @@ @@   @@     @@@  @@ @@    @@@ @@@
//   @@@@@ @@@@@  @@@@  @@@@@   @@@@@ @@@@@  @@   @@      @   @@ @@@@@ @@   @@
//
//  Editieren von Klebepunkten an den Objekten (Klebepunkte fuer Verbinder)
//
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////

class SVX_DLLPUBLIC SdrGlueEditView: public SdrPolyEditView
{
	void ImpClearVars();

	// Markierte Klebepunkte kopieren und anstelle der alten markieren
	void ImpCopyMarkedGluePoints();
	typedef void (*PGlueDoFunc)(SdrGluePoint&, const SdrObject* pObj, const void*, const void*, const void*, const void*, const void*);
	typedef void (*PGlueTrFunc)(Point&, const void*, const void*, const void*, const void*, const void*);
	void ImpDoMarkedGluePoints(PGlueDoFunc pDoFunc, BOOL bConst, const void* p1=NULL, const void* p2=NULL, const void* p3=NULL, const void* p4=NULL, const void* p5=NULL);
	void ImpTransformMarkedGluePoints(PGlueTrFunc pTrFunc, const void* p1=NULL, const void* p2=NULL, const void* p3=NULL, const void* p4=NULL, const void* p5=NULL);

protected:
	// #i71538# make constructors of SdrView sub-components protected to avoid incomplete incarnations which may get casted to SdrView
	SdrGlueEditView(SdrModel* pModel1, OutputDevice* pOut = 0L);
	virtual ~SdrGlueEditView();

public:
	// Durch den Parameter nThisEsc uebergibt man die Richtung, die man
	// checken bzw. setzen/loeschen will.
	// Moegliche Werte fuer nThisEsc sind z.Zt.
	// SDRESC_LEFT, SDRESC_RIGHT, SDRESC_TOP und SDRESC_BOTTOM
	TRISTATE IsMarkedGluePointsEscDir(USHORT nThisEsc) const;
	void SetMarkedGluePointsEscDir(USHORT nThisEsc, BOOL bOn);
	BOOL IsSetMarkedGluePointsEscDirPossible() const { return !IsReadOnly() && HasMarkedGluePoints(); }

	// Checken/setzen, ob die Klebepunktpositionen relativ zur
	// Objektgroesse sind (Percent=TRUE) oder nicht (Percent=FALSE)
	TRISTATE IsMarkedGluePointsPercent() const;
	void SetMarkedGluePointsPercent(BOOL bOn);
	BOOL IsSetMarkedGluePointsPercentPossible() const { return !IsReadOnly() && HasMarkedGluePoints(); }

	// bVert=FALSE: Horizontales Alignment checken/setzen
	//      SDRHORZALIGN_CENTER
	//      SDRHORZALIGN_LEFT
	//      SDRHORZALIGN_RIGHT
	//      SDRHORZALIGN_DONTCARE (nur bei Get())
	// bVert=TRUE: Vertikales Alignment checken/setzen
	//      SDRVERTALIGN_CENTER
	//      SDRVERTALIGN_TOP
	//      SDRVERTALIGN_BOTTOM
	//      SDRVERTALIGN_DONTCARE (nur bei Get())
	USHORT GetMarkedGluePointsAlign(BOOL bVert) const;
	void SetMarkedGluePointsAlign(BOOL bVert, USHORT nAlign);
	BOOL IsSetMarkedGluePointsAlignPossible() const { return !IsReadOnly() && HasMarkedGluePoints(); }

	// Alle merkierten Klebepunkte entfernen
	void DeleteMarkedGluePoints();
	BOOL IsDeleteMarkedGluePointsPossible() const;

	void MoveMarkedGluePoints  (const Size& rSiz, bool bCopy=false);
	void ResizeMarkedGluePoints(const Point& rRef, const Fraction& xFact, const Fraction& yFact, bool bCopy=false);
	void RotateMarkedGluePoints(const Point& rRef, long nWink, bool bCopy=false);
	void MirrorMarkedGluePoints(const Point& rRef1, const Point& rRef2, BOOL bCopy=FALSE);
	void MirrorMarkedGluePointsHorizontal(BOOL bCopy=FALSE);
	void MirrorMarkedGluePointsVertical(BOOL bCopy=FALSE);
	void ShearMarkedGluePoints(const Point& rRef, long nWink, BOOL bVShear=FALSE, BOOL bCopy=FALSE);
	void CrookMarkedGluePoints(const Point& rRef, const Point& rRad, SdrCrookMode eMode, BOOL bVertical=FALSE, BOOL bNoContortion=FALSE, BOOL bCopy=FALSE);
	void DistortMarkedGluePoints(const Rectangle& rRef, const XPolygon& rDistortedRect, BOOL bNoContortion=FALSE, BOOL bCopy=FALSE);

	void AlignMarkedGluePoints(SdrHorAlign eHor, SdrVertAlign eVert);
};

#endif //_SVDGLEV_HXX

